@@ -22,8 +22,8 @@ stack::call::call() : TimerQueue::event(Timer::reset), segments()
 {
 	arm(stack::resetTimeout());
 	count = 0;
-	fwdmask = 0;
-	forwarding = FWD_IGNORE;
+	forwarding = diverting = NULL;
+	answering = 4;	// should set from cfna timer...
 	invited = ringing = ringbusy = unreachable = 0;
 	phone = false;
 	expires = 0l;
@@ -58,6 +58,8 @@ void stack::call::joinLocked(session *join)
 	if(!map->active)
 		time(&map->active);
 
+	// once we have joined, there is no more forwarding...
+	forwarding = diverting = NULL;
 	target = join;
 	while(sp) {
 		s = &(sp->sid);
@@ -155,21 +157,35 @@ void stack::call::closingLocked(session *s)
 	if(invited)
 		--invited;
 
-	if(!invited)
-		disconnectLocked();
+	if(!invited) {
+		if(!stack::forward(this))	
+			disconnectLocked();
+		if(state == RINGING) {
+			arm(stack::ringTimeout());
+			reply_source(SIP_CALL_IS_BEING_FORWARDED);
+		}
+	}
 }
 
 void stack::call::reply_source(int error)
 {
 	osip_message_t *reply = NULL;
+
+	debug(3, "sip: sending source reply %d", error);
+
+	if(error == SIP_CALL_IS_BEING_FORWARDED && answering)
+		answering = 4;
+
 	eXosip_lock();
 	eXosip_call_build_answer(source->tid, error, &reply);
 	if(reply != NULL) {
 		stack::siplog(reply);
 		eXosip_call_send_answer(source->tid, error, reply);
 	}
-	else
+	else {
+		debug(3, "sip: source reply %d failed", error);
 		eXosip_call_send_answer(source->tid, SIP_BAD_REQUEST, NULL);
+	}
 	eXosip_unlock();
 }
 
@@ -597,7 +613,20 @@ void stack::call::busy(thread *thread, session *s)
 	case INITIAL:
 	case RINGING:
 	case RINGBACK:
-		if(!ringing && ringbusy) {
+		// we goto busy in this special case, otherwise stack::close handles na
+		if(!ringing && ringbusy && invited == 1) {
+			if(forwarding) {
+				forwarding = "busy";
+				if(stack::forward(this)) {
+					ringbusy = 0;
+					if(state == RINGING)
+						arm(stack::ringTimeout());
+					Mutex::release(this);
+					if(state == RINGING)
+						reply_source(SIP_CALL_IS_BEING_FORWARDED);
+					return;
+				}
+			}
 			set(BUSY, 'b', "busy");
 			disconnectLocked();
 		}
@@ -618,9 +647,13 @@ void stack::call::trying(thread *thread)
 	// turn-over state and set timer to wait for all invites to become
 	// busy or one or more to start ringing...
 	//
-	if(state == INITIAL) 
-		// we cannot reply_source because build always fails!
+	if(state == INITIAL) {
+		debug(3, "sip: sending source reply %d", SIP_TRYING);
+		// we cannot use reply_source because build always fails!
+		eXosip_lock();
 		eXosip_call_send_answer(source->tid, SIP_TRYING, NULL);
+		eXosip_unlock();
+	}
 
 	Mutex::protect(this);
 	set(TRYING, 't', "trying");
@@ -638,11 +671,24 @@ void stack::call::expired(void)
 	case HOLDING:	// hold-recall timer expired...
 
 	case RINGING:	// re-generate ring event to origination...
+					// also controls call-forward no-answer timing...
+			if(answering == 1 && forwarding) {
+				forwarding = "na";
+				if(stack::forward(this)) {
+					arm(stack::ringTimeout());
+					Mutex::release(this);
+					reply_source(SIP_CALL_IS_BEING_FORWARDED);
+					return;
+				}
+				disconnectLocked();
+				break;
+			}
+			if(answering)
+				--answering;
 			arm(stack::ringTimeout());	
 			Mutex::release(this);
 			reply_source(SIP_RINGING);
 			return;
-			
 	case REDIRECT:	// FIXME: add refer select of next segment if list....
 	
 	case RINGBACK:
