@@ -447,8 +447,10 @@ void registry::reload(service *cfg)
 {
 	assert(cfg != NULL);
 
+	static const char *oldrealm = "-";
 	const char *key = NULL, *value;
 	linked_pointer<service::keynode> sp = cfg->getList("registry");
+	fsys_t fd;
 
 	while(is(sp)) {
 		key = sp->getId();
@@ -493,7 +495,20 @@ void registry::reload(service *cfg)
 	memset(contacts, 0, sizeof(LinkedObject *) * keysize);
 	memset(publishing, 0, sizeof(LinkedObject *) * keysize);
 	memset(addresses, 0, sizeof(LinkedObject *) * keysize);
-	process::errlog(INFO, "realm %s", realm);
+
+	Mutex::protect(&oldrealm);
+	if(!String::equal(realm, oldrealm)) {
+		process::errlog(INFO, "realm is %s", realm);
+		oldrealm = realm;
+#ifndef	_MSWINDOWS_
+		fsys::open(fd, "/tmp/siprealm", fsys::ACCESS_WRONLY);
+		if(is(fd)) {
+			fsys::write(fd, realm, strlen(realm));		
+			fsys::close(fd);
+		}
+#endif
+	}
+	Mutex::release(&oldrealm);
 }
 
 unsigned registry::getEntries(void)
@@ -1003,6 +1018,7 @@ unsigned registry::mapped::setTarget(Socket::address& target_addr, time_t lease,
 
 	if(!tp) {
 		tp = new target;
+		time(&tp->created);
 		tp->enlist(&internal.targets);
 		count = 1;
 		tp->status = registry::target::READY;
@@ -1184,7 +1200,7 @@ unsigned registry::mapped::addTarget(Socket::address& target_addr, time_t lease,
 	len = Socket::getlen(ai);
 	time(&now);
 	while(tp) {
-		if(expires < now)
+		if(tp->expires < now)
 			expired = *tp;
 		if(Socket::equal((struct sockaddr *)(&tp->address), ai))
 			break;
@@ -1201,6 +1217,8 @@ unsigned registry::mapped::addTarget(Socket::address& target_addr, time_t lease,
 			--count;
 			delete expired;
 		}
+		if(tp->expires < now)
+			time(&tp->created);
 		tp->expires = lease;
 		String::set(tp->contact, sizeof(tp->contact), target_contact);
 		locking.share();
@@ -1222,6 +1240,7 @@ unsigned registry::mapped::addTarget(Socket::address& target_addr, time_t lease,
 			delete origin;
 		++count;
 	}
+	time(&expired->created);
 	expired->expires = lease;
 	memcpy(&expired->address, ai, len);
 	stack::getInterface((struct sockaddr *)(&expired->iface), (struct sockaddr *)(&expired->address));
@@ -1262,6 +1281,7 @@ unsigned registry::mapped::setTargets(Socket::address& target_addr)
 		len = Socket::getlen(al->ai_addr);
 
 		tp = new target;
+		time(&tp->created);
 		memcpy(&tp->address, al->ai_addr, len);
 		memcpy(&contact, &tp->address, len);
 		remote[0] = 0;
