@@ -48,6 +48,8 @@ using namespace UCOMMON_NAMESPACE;
 
 class thread;
 
+typedef enum {EXTERNAL, LOCAL, PUBLIC, ROUTED, FORWARDED} destination_t;
+
 class __LOCAL registry : private service::callback, private mapped_array<MappedRegistry> 
 { 
 public: 
@@ -102,7 +104,8 @@ public:
 			target *getTarget(void);
 		} index;
 		struct sockaddr_internet address;
-		struct sockaddr_internet iface; 
+		struct sockaddr_internet iface;
+		time_t created; 
 		volatile time_t expires;
 		enum {READY, BUSY, AWAY, DND} status;
 		char contact[MAX_URI_SIZE]; 
@@ -284,11 +287,11 @@ private:
 		call();
 
 		state_t state;
-		unsigned fwdmask;				// forwarding mask in effect...
-		char forward[MAX_IDENT_SIZE];	// ref id for forwarding...
+		struct sockaddr_internet iface;	// source interface...
+		char forward[MAX_USERID_SIZE];	// ref id for forwarding...
+		char divert[MAX_USERID_SIZE];	// used in forward management
 		char dialed[MAX_IDENT_SIZE];	// user or ip address...
 		char subject[MAX_URI_SIZE];		// call subject
-		char refer[MAX_IDENT_SIZE];		// used in forward management
 		rtpproxy *rtp;
 
 		void reply_source(int error);
@@ -317,7 +320,9 @@ private:
 		session *target;
 		segment *select;
 		MappedCall *map;
-		enum {FWD_IGNORE, FWD_NA, FWD_BUSY, FWD_DND, FWD_AWAY, FWD_ALL} forwarding;
+		const char *forwarding;
+		const char *diverting;
+		unsigned answering;		// answer ring supervision...
 		unsigned count;			// total open segments
 		unsigned invited;		// pending segments with invites
 		unsigned ringing;		// number of ringing segments
@@ -339,6 +344,8 @@ private:
 	void update(void);
 	void modify(void);
 
+	static void divert(stack::call *cr, struct sockaddr_internet *addr, osip_message_t *msg);
+
 	unsigned threading, priority;
 	size_t stacksize;
 
@@ -385,6 +392,10 @@ public:
 	static int getDialog(session *session);
 	static void release(MappedCall *map);
 	static MappedCall *get(void);
+	static bool forward(stack::call *cr);
+	static bool assign(stack::call *cr, unsigned count);
+	static void inviteRemote(stack::session *session, const char *uri);
+	static void inviteLocal(stack::session *session, registry::mapped *rr, destination_t dest);
 
 	inline static timeout_t ringTimeout(void)
 		{return stack::sip.ring_timer;};
@@ -424,7 +435,6 @@ public:
 	static void getProvision(const char *id, usernode& user);
 	static void getDialing(const char *id, usernode& user);
 	static keynode *getConfig(void);
-	static unsigned getForwarding(const char *id);
 	static cidr *getPolicy(struct sockaddr *addr);
 	static bool isLocal(struct sockaddr *addr);
 	static void release(cidr *access);
@@ -432,7 +442,6 @@ public:
 	static void release(usernode& user);
 	static void reload(const char *uid);
 	static Socket::address *getContact(const char *id);
-	static unsigned forwarding(keynode *node);
 	static void plugins(const char *argv0, const char *names);
 	static void usage(void);
 	static void version(void);
@@ -523,13 +532,12 @@ private:
 	const char *via_host;
 	unsigned via_hops;
 	unsigned via_port;
-	rtpproxy::session proxyinfo;
+	destination_t destination;
 
 	char *sip_realm;
 	osip_proxy_authenticate_t *proxy_auth;
 	osip_www_authenticate_t *www_auth;
 
-	enum {EXTERNAL, LOCAL, PUBLIC, ROUTED, FORWARDED} destination;
 	enum {CALL, MESSAGE, REGISTRAR, NONE} authorizing;
 
 	thread();
@@ -538,8 +546,6 @@ private:
 
 	void send_reply(int error);
 	void expiration(void);
-	void inviteRemote(stack::session *session, const char *uri);
-	void inviteLocal(stack::session *session, registry::mapped *rr);
 	void invite(void);
 	void identify(void);
 	bool getsource(void);
@@ -560,7 +566,6 @@ private:
 
 public:
 	static void shutdown(void);
-	static bool assign(stack::call *cr, unsigned count);
 };
 
 END_NAMESPACE
